-- ============================================================================
-- == HalonaTools - A collection of functions used to help users monitor     ==
-- ==               their buffs.                                             ==
-- ============================================================================

HTBuffMonitor               = LibStub("AceAddon-3.0"):NewAddon("HTBuffMonitor", "AceConsole-3.0", "AceEvent-3.0")
local L                     = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local foodBuffDispensers    = {} -- Holds spell IDs for feasts, banquets etc.
local flaskDispensers       = {} -- Holds spell IDs for cauldrons etc.
local healthStoneDispensers = {} -- Holds spell IDs for candy-mountains etc.
local manaFoodDispensers    = {} -- Holds spell IDs for ritual of refreshment etc.
local repairBots            = {} -- Holds spell IDs for repair bots.

-- Frame required to permit tooltips to be scanned for temp-buffs like poisons.
local TooltipScanner = CreateFrame('GameTooltip', 'TooltipScanner', WorldFrame, 'GameTooltipTemplate')

-- Some checks, triggered by events, should only happen once until reset event.
local isPrimedCombatCheck = true

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBuffMonitor:OnInitialize()
end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBuffMonitor:OnEnable()

  -- Initialise the Buff Announcer collections.
  HTBuffMonitor:InitBuffAnnouncerCollections()

  -- Subscribe to events that are used to trigger buff checks.
  self:RegisterEvent("READY_CHECK")
  self:RegisterEvent("PLAYER_REGEN_ENABLED")
  self:RegisterEvent("PLAYER_REGEN_DISABLED")
  self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

  -- Declare the tooltip scanner's owner. Used to check weapon buffs.
  TooltipScanner:SetOwner(WorldFrame, "ANCHOR_NONE")

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBuffMonitor:OnDisable()
end

-- ============================================================================
-- == InitBuffAnnouncerCollections()                                         ==
-- ==                                                                        ==
-- == This function initialises the BuffAnnouncer data collections which     ==
-- == contain the various lists of group-buff or utility dispensing items.   ==
-- ==                                                                        ==
-- == E.g.: Buff Food, Flasks, Health-Stones, Mana-Food and Repair-Bots.     ==
-- ============================================================================

function HTBuffMonitor:InitBuffAnnouncerCollections()

  HTBuffMonitor:InitFoodBuffDispensers()
  HTBuffMonitor:InitFlaskDispensers()
  HTBuffMonitor:InitHealthStoneDispensers()
  HTBuffMonitor:InitManaFoodDispensers()
  HTBuffMonitor:InitRepairBots()

end

-- ============================================================================
-- == InitFoodBuffDispensers()                                               ==
-- ==                                                                        ==
-- == This function initialises the foodBuffDispensers table which is used   ==
-- == by the BuffAnnouncer feature to recognise when a food buff item has    ==
-- == been deployed.                                                         ==
-- ==                                                                        ==
-- == E.g.: Feasts, Banquets etc.                                            ==
-- ============================================================================

function HTBuffMonitor:InitFoodBuffDispensers()

  -- [Mists of Pandaria] - "Great Banquets" - For 25 Players.
  table.insert(foodBuffDispensers, tonumber("126504")) -- Great Banquet of the Brew
  table.insert(foodBuffDispensers, tonumber("126494")) -- Great Banquet of the Grill
  table.insert(foodBuffDispensers, tonumber("126502")) -- Great Banquet of the Oven
  table.insert(foodBuffDispensers, tonumber("126498")) -- Great Banquet of the Pot
  table.insert(foodBuffDispensers, tonumber("126500"))  -- Great Banquet of the Steamer
  table.insert(foodBuffDispensers, tonumber("126496"))  -- Great Banquet of the Wok

  -- [Mists of Pandaria] - "Banquets" - For 10 Players.
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Brew
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Grill
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Oven
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Pot
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Steamer
  -- table.insert(foodBuffDispensers, tonumber("XXXXXX"))  -- Banquet of the Wok

  -- [Cataclysm] - "Feasts"
  table.insert(foodBuffDispensers, tonumber("87644"))  -- Seafood Magnifique Feast
  table.insert(foodBuffDispensers, tonumber("87643"))  -- Broiled Dragon Feast
  table.insert(foodBuffDispensers, tonumber("87915"))  -- Goblin Barbecue Feast

  -- [Wrath of the Lich King] - "Feasts"
  table.insert(foodBuffDispensers, tonumber("57426"))  -- Fish Feast
  table.insert(foodBuffDispensers, tonumber("16936"))  -- {unknown}

end

-- ============================================================================
-- == InitFlaskDispensers()                                                  ==
-- ==                                                                        ==
-- == This function initialises the foodBuffDispensers table which is used   ==
-- == by the BuffAnnouncer feature to recognise when a flask dispensing item ==
-- == has been deployed.                                                     ==
-- ==                                                                        ==
-- == E.g.: Cauldrons etc.                                                   ==
-- ============================================================================

function HTBuffMonitor:InitFlaskDispensers()

  -- [Cataclysm] - "Cauldrons"
  table.insert(flaskDispensers, tonumber("92712"))  -- Big Cauldron of Battle
  table.insert(flaskDispensers, tonumber("92649"))  -- Cauldron of Battle

end

-- ============================================================================
-- == InitHealthStoneDispensers()                                            ==
-- ==                                                                        ==
-- == This function initialises the foodBuffDispensers table which is used   ==
-- == by the BuffAnnouncer feature to recognise when a health-stone          ==
-- == dispensing item has been deployed.                                     ==
-- ==                                                                        ==
-- == E.g.: Soulwells (aka: Candy Mountains) etc.                            ==
-- ============================================================================

function HTBuffMonitor:InitHealthStoneDispensers()

  -- [Common] - "Ritual of Souls"
  table.insert(healthStoneDispensers, tonumber("29893"))  -- Ritual of Souls

end

-- ============================================================================
-- == InitManaFoodDispensers()                                               ==
-- ==                                                                        ==
-- == This function initialises the foodBuffDispensers table which is used   ==
-- == by the BuffAnnouncer feature to recognise when a mana-food dispensing  ==
-- == item has been deployed.                                                ==
-- ==                                                                        ==
-- == E.g.: Ritual of Refreshment etc.                                       ==
-- ============================================================================

function HTBuffMonitor:InitManaFoodDispensers()

  -- [Common] - "Ritual of Refreshment"
  table.insert(manaFoodDispensers, tonumber("43987"))  -- Ritual of Refreshment

end

-- ============================================================================
-- == InitRepairBots()                                                       ==
-- ==                                                                        ==
-- == This function initialises the foodBuffDispensers table which is used   ==
-- == by the BuffAnnouncer feature to recognise when a repair bot item has   ==
-- == been deployed.                                                         ==
-- ==                                                                        ==
-- == E.g.: XXX, YYY, ZZZ.                                                   ==
-- ============================================================================

function HTBuffMonitor:InitRepairBots()

  -- [Common] - "Repair Bots"
  table.insert(repairBots, tonumber("67826"))  -- Jeeves
  table.insert(repairBots, tonumber("54711"))  -- Scrapbot
  table.insert(repairBots, tonumber("44389"))  -- Field Repair Bot 110G
  table.insert(repairBots, tonumber("22700"))  -- Field Repair Bot 74A

  -- TODO: Any other repair/vendors? (Mammoths?, Squires?, Others??)

end

-- ============================================================================
-- == DoMissingBuffCheck(characterName, channelName)                         ==
-- ==                                                                        ==
-- == This function will check the named character for missing buffs.        ==
-- ============================================================================

function HTBuffMonitor:DoMissingBuffCheck(characterName, channelName)

  local hasSuitableFlask             = HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)
  local hasSuitableBattleElixir      = HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)
  local hasSuitableGuardianElixir    = HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)
  local hasSuitableFoodBuff          = HTBuffMonitor:IsBuffedWithSuitableFood(characterName)
  local hasSuitableDamagePoisonBuff  = HTBuffMonitor:IsBuffedWithSuitableRogueDamagePoison(characterName)
  local hasSuitableUtilityPoisonBuff = HTBuffMonitor:IsBuffedWithSuitableRogueUtilityPoison(characterName)

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a flask or two elixirs activated.          --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFlask then

    -- == ---------------------------------------------------------------------
    -- == -- Has guardian elixir but suitable battle elixir missing.         --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and     hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-battle-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- Has battle elixir but suitable guardian elixir missing.         --
    -- == ---------------------------------------------------------------------
    if      hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-guardian-elixir"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

    -- == ---------------------------------------------------------------------
    -- == -- No flask, no battle elixir, no guardian elixir.                 --
    -- == ---------------------------------------------------------------------
    if  not hasSuitableBattleElixir
    and not hasSuitableGuardianElixir then
      if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-no-flask-or-elixirs"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end

  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has a food-buff activated.                     --
  -- == -----------------------------------------------------------------------
  if not hasSuitableFoodBuff then
    if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
      HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-food"], "CHARACTER_NAME", characterName), channelName, nil, nil)
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character has suitable rogue poisons activated.          --
  -- == -----------------------------------------------------------------------
  if UnitClass(characterName) == L["all-data-classname-rogue"] then
    if not hasSuitableDamagePoisonBuff then
      if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-weapon-poison-damage"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end
  end

  if UnitClass(characterName) == L["all-data-classname-rogue"] then
    if not hasSuitableUtilityPoisonBuff then
      if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
        HTConsole:SendChatMessage(string.gsub(L["bfm-missing-buff-weapon-poison-utility"], "CHARACTER_NAME", characterName), channelName, nil, nil)
      end
    end
  end

  -- == -----------------------------------------------------------------------
  -- == -- Check if character buffs are nearing expiry.                      --
  -- == -----------------------------------------------------------------------
  if HTConfig:GetIsEnabledBuffMonitorExpiryTimeCheck(nil) then
     HTBuffMonitor:CheckExpiryTimes(characterName, channelName)
  end

end

-- ============================================================================
-- == IsCharacterBuffedWithSuitableFlask(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable flask activated.                                              ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFlask(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFlasksArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableBattleElixir(characterName)                        ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable battle elixir activated.                                      ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableBattleElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableBattleElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableGuardianElixir(characterName)                      ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable guardian elixir activated.                                    ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableGuardianElixir(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableGuardianElixirsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableFood(characterName)                                ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable food buff activated.                                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableFood(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableFoodArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableRogueDamagePoison(characterName)                   ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable rogue damage poison buff activated.                           ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableRogueDamagePoison(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableRogueDamagePoisonsArray())

end

-- ============================================================================
-- == IsBuffedWithSuitableRogueUtilityPoison(characterName)                  ==
-- ==                                                                        ==
-- == This function will return true if the character being tested has a     ==
-- == suitable rogue utility poison buff activated.                          ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWithSuitableRogueUtilityPoison(characterName)

  return HTBuffMonitor:IsBuffedWith(characterName, HTBuffMonitor:GetSuitableRogueUtilityPoisonsArray())

end

-- ============================================================================
-- == IsBuffedWith(characterName, suitableBuffs)                             ==
-- ==                                                                        ==
-- == This function performs a check on the character identified by the      ==
-- == supplied parameter to determine if that character is currently buffed  ==
-- == with any of the buffs detailed in the supplied 'suitableBuffs' table.  ==
-- ============================================================================

function HTBuffMonitor:IsBuffedWith(characterName, suitableBuffs)

  -- == ----------------------------------------------------------------------
  -- == -- Perform the test.                                                --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName = UnitBuff(characterName, b)
    if buffName then
      for i,v in ipairs(suitableBuffs) do
        if suitableBuffs[i] == buffName then
          return true
        end
      end
    end
  end

  return false

end

-- ============================================================================
-- == CheckExpiryTimes(characterName, channelName)                           ==
-- ==                                                                        ==
-- == This function simply checks the expiry times of the buff-types that    ==
-- == HalonaTools monitors and reports any buffs that will expire within a   ==
-- == prescribed time-limit to the nominated channel.                        ==
-- ============================================================================

function HTBuffMonitor:CheckExpiryTimes(characterName, channelName)

  -- HTConsole:Debug("HTBuffMonitor:CheckExpiryTimes("..characterName..", "..channelName..")")

  -- == ----------------------------------------------------------------------
  -- == -- Setup the 'suitable' buff array and other required properties.   --
  -- == ----------------------------------------------------------------------
  local suitableFlasksArray              = HTBuffMonitor:GetSuitableFlasksArray()
  local suitableBattleElixirsArray       = HTBuffMonitor:GetSuitableBattleElixirsArray()
  local suitableGuardianElixirsArray     = HTBuffMonitor:GetSuitableGuardianElixirsArray()
  local suitableFoodArray                = HTBuffMonitor:GetSuitableFoodArray()
  local suitableRogueDamagePoisonsArray  = HTBuffMonitor:GetSuitableRogueDamagePoisonsArray()
  local suitableRogueUtilityPoisonsArray = HTBuffMonitor:GetSuitableRogueUtilityPoisonsArray()

  local timeLimitMinutes                 = HTConfig:GetBuffMonitorExpiryTimeLimit(nil)
  local suitableBuffsArray               = {}

  if HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFlasksArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableBattleElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableGuardianElixirsArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckFood(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableFoodArray)
  end

  if HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) then
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableRogueDamagePoisonsArray)
    suitableBuffsArray = HTData:JoinTables(suitableBuffsArray, suitableRogueUtilityPoisonsArray)
  end

  -- == ----------------------------------------------------------------------
  -- == -- Perform the tests.                                               --
  -- == ----------------------------------------------------------------------
  for b = 1, 64 do
    local buffName, buffRank, buffIcon, buffCount, buffDispelType, buffDuration, buffExpires, buffCaster, buffIsStealable = UnitBuff(characterName, b)
    if buffName then
      if buffDuration then
        for i,v in ipairs(suitableBuffsArray) do
          if suitableBuffsArray[i] == buffName then

            remainingTime = HTData:SafeNumber(tonumber(format("%.2f",-1*(GetTime()-buffExpires)/60)))

            -- HTConsole:Debug("Buff: "..buffName)
            -- HTConsole:Debug("Remaining Time: "..remainingTime..", timeLimitMinutes: "..HTData:SafeNumber(timeLimitMinutes))

            if remainingTime < timeLimitMinutes then
              local messageText = string.gsub(L["bfm-short-buff-duration-announce"], "CHARACTER_NAME", characterName)
              messageText       = string.gsub(messageText, "BUFF_NAME", buffName)
              messageText       = string.gsub(messageText, "TIME_LIMIT", timeLimitMinutes)

              HTConsole:SendChatMessage(messageText, channelName, nil, nil)

            end
          end
        end
      end
    end
  end

end

-- ============================================================================
-- == GetSuitableFlasksArray()                                               ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == flasks with respect to PVE raid usage.                                 ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFlasksArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105689))  -- Flask of Spring Blossoms
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105691))  -- Flask of the Warm Sun
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105693))  -- Flask of Falling Leaves
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105694))  -- Flask of the Earth
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105696))  -- Flask of Winter's Bite
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(127230))  -- Crystal of Insanity / Visions of Insanity
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79469))  -- Flask of Steelskin.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79470))  -- Flask of the Draconic Mind.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79471))  -- Flask of the Winds.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79472))  -- Flask of Titanic Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(94160))  -- Flask of Flowing Water.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFlasks(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53760))  -- Flask of Endless Rage.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53755))  -- Flask of the Frost Wyrm.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(54212))  -- Flask of Pure Mojo.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53758))  -- Flask of Stoneblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(62380))  -- Lesser Flask of Resistance.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableBattleElixirsArray()                                        ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == battle elixirs with respect to PVE raid usage.                         ==
-- ============================================================================

function HTBuffMonitor:GetSuitableBattleElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsBattleElixirs(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105682))  -- Mad Hozen Elixir
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105683))  -- Elixir of Weaponry
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105684))  -- Elixir of the Rapids
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105685))  -- Elixir of Peace
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105686))  -- Elixir of Perfection
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105688))  -- Monk's Elixir
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79477)) -- Elixir of the Cobra.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79481)) -- Elixir of Impossible Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79632)) -- Elixir of Mighty Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79635)) -- Elixir of the Master.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79468)) -- Ghost Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79474)) -- Elixir of the Naga.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathBattleElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60345)) -- Armor Piercing.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60341)) -- Deadly Strikes.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60344)) -- Expertise.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53749)) -- Guru's Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60346)) -- Lightning Speed.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(28497)) -- Mighty Agility.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53748)) -- Mighty Strength.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(33721)) -- Spellpower Elixir.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53746)) -- Wrath Elixir.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableGuardianElixirsArray()                                      ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == guardian elixirs with respect to PVE raid usage.                       ==
-- ============================================================================

function HTBuffMonitor:GetSuitableGuardianElixirsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsGuardianElixirs(nil) then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105681))  -- Mantid Elixir
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(105687))  -- Elixir of Mirrors
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79480)) -- Elixir of Deep Earth.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(79631)) -- Prismatic Elixir.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathGuardianElixirs() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60340)) -- Accuracy.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53763)) -- Wrath - Elixir of Protection.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53747)) -- Wrath - Elixir of Spirit.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60343)) -- Wrath - Mighty Defense.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53751)) -- Wrath - Mighty Fortitude.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(53764)) -- Wrath - Mighty Mageblood.
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(60347)) -- Wrath - Mighty Thoughts.
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableFoodArray()                                                 ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == food related buffs with respect to PVE raid usage.                     ==
-- ============================================================================

function HTBuffMonitor:GetSuitableFoodArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsFood(nil) then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????))  -- Well Fed ?????
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataFood() then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????))  -- Well Fed ?????
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathFood() then
    table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(35272)) -- Well Fed
  end

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableRogueDamagePoisonsArray()                                   ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == rogue damage poisons with respect to PVE raid usage.                   ==
-- ============================================================================

function HTBuffMonitor:GetSuitableRogueDamagePoisonsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsPoisons(nil) then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataPoisons() then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathPoisons() then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data common to all expansions.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(2818)) -- Deadly Poison.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(8679)) -- Wound Poison.

  return suitableBuffs

end

-- ============================================================================
-- == GetSuitableRogueUtilityPoisonsArray()                                  ==
-- ==                                                                        ==
-- == This function simply returns the currently valid array of suitable     ==
-- == rogue utility poisons with respect to PVE raid usage.                  ==
-- ============================================================================

function HTBuffMonitor:GetSuitableRogueUtilityPoisonsArray()

  -- == -----------------------------------------------------------------------
  -- == -- Suitable buffs are identified by their spell-IDs.                 --
  -- == -----------------------------------------------------------------------
  local suitableBuffs = {}

  -- Add data for the Mists expansion.
  if HTConfig:GetIsEnabledBuffMonitorMistsPoisons(nil) then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data for the Cataclysm expansion.
  if HTConfig:GetIsEnabledBuffMonitorCataPoisons() then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data for the Wrath expansion.
  if HTConfig:GetIsEnabledBuffMonitorWrathPoisons() then
    -- table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(?????)) -- XYZ Poison.
  end

  -- Add data common to all expansions.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(112974)) -- Leeching Poison.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(87972))  -- Paralytic Poison.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(3408))   -- Crippling Poison.
  table.insert(suitableBuffs, HTBuffMonitor:GetSpellNameFromSpellInfo(5761))   -- Mind-Numbing Poison.

  return suitableBuffs

end

-- ============================================================================
-- == GetSpellNameFromSpellInfo(spellId)                                     ==
-- ==                                                                        ==
-- == This function simply returns the name of a spell when supplied with    ==
-- == a spellId by using the standard API function GetSpellInfo(spellId) and ==
-- == retrieving the name from the data returned.                            ==
-- ============================================================================

function HTBuffMonitor:GetSpellNameFromSpellInfo(spellId)

   spellName, spellRank, spellIcon, spellPowerCost, spellIsFunnel, spellPowerType, spellCastingTime, spellMinRange, spellMaxRange = GetSpellInfo(spellId)

   spellName = HTData:SafeString(spellName)

   return spellName

end

-- ============================================================================
-- == READY_CHECK(sender)                                                    ==
-- ==                                                                        ==
-- == This function is triggered when a ready check is fired.                ==
-- ============================================================================

function HTBuffMonitor:READY_CHECK(sender)

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnReadyCheck(nil) then
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-01"], "CHAT", nil, nil)
    HTBuffMonitor:DoSelfCheck()
    HTConsole:SendChatMessage(L["bfm-readycheck-msg-02"], "CHAT", nil, nil)
    HTBuffMonitor:DoRaidCheck("RAID")
  end

end

-- ============================================================================
-- == PLAYER_REGEN_DISABLED()                                                ==
-- ==                                                                        ==
-- == This function is triggered when the player enters combat.              ==
-- ============================================================================

function HTBuffMonitor:PLAYER_REGEN_DISABLED()

  HTConfig:SetIsPlayerInCombat(true, true)

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnCombatStart(nil) then
    if isPrimedCombatCheck == true then
      HTBuffMonitor:DoRaidCheck("CHAT")
      isPrimedCombatCheck = false
    end
  end

end

-- ============================================================================
-- == PLAYER_REGEN_ENABLED()                                                 ==
-- ==                                                                        ==
-- == This function is triggered when the player leaves combat.              ==
-- ============================================================================

function HTBuffMonitor:PLAYER_REGEN_ENABLED()

  HTConfig:SetIsPlayerInCombat(false, false)

  -- Ensures tasks may be performed when combat begins without repeating.
  isPrimedCombatCheck = true

end

-- ============================================================================
-- == COMBAT_LOG_EVENT_UNFILTERED(...)                                       ==
-- ==                                                                        ==
-- == This function parses the combat log events when they fire,             ==
-- == to determine whether they represent an event occurring that the addon  ==
-- == is expected to respond to.                                             ==
-- ============================================================================

function HTBuffMonitor:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, subevent, hideCaster, srcGUID, srcname, srcflags, dstGUID, dstname, dstflags, spellID, spellname, spellschool, extraspellID, extraspellname, extraspellschool, auratype)

  -- Attempt to capture some useful information using debug mode.
  if HTConfig:GetIsEnabledDebugMode(nil) == true then
    if HTConfig:GetIsPlayerInCombat(nil) == false then
      local characterName = string.lower(HTData:SafeString(srcname))

      -- spellID, spellname, spellschool, extraspellID, extraspellname, extraspellschool, auratype
      if characterName == "broliant" or characterName == "kenjymin" or characterName == "vanhelgen" or characterName == "koyaanis" then
        HTConsole:Debug(HTConsole:GetTitleBar());
        HTConsole:Debug("Character: [" ..        HTData:SafeString(srcname)          .. "]")
        HTConsole:Debug("subevent: [" ..         HTData:SafeString(subevent)         .. "]")
        HTConsole:Debug("spellID: [" ..          HTData:SafeString(spellID)          .. "]")
        HTConsole:Debug("spellname: [" ..        HTData:SafeString(spellname)        .. "]")
        HTConsole:Debug("spellschool: [" ..      HTData:SafeString(spellschool)      .. "]")
        HTConsole:Debug("extraspellID: [" ..     HTData:SafeString(extraspellID)     .. "]")
        HTConsole:Debug("extraspellname: [" ..   HTData:SafeString(extraspellname)   .. "]")
        HTConsole:Debug("extraspellschool: [" .. HTData:SafeString(extraspellschool) .. "]")
        HTConsole:Debug("auratype: [" ..         HTData:SafeString(auratype)         .. "]")
        HTConsole:Debug(HTConsole:GetTitleBar());
      end
    end
  end

  -- Perform the following ONLY when the player is NOT in combat.
  if HTConfig:GetIsPlayerInCombat(nil) == false then

    -- Ensure required parameters have been returned, otherwise exit immediately.
    -- *NB: The COMBAT_LOG_EVENT_UNFILTERED function signature may vary from
    -- time to time - therefore, the 'spellschool' parameter currently holds the
    -- value we consider to be the 'spellID'.
    if not subevent or not spellschool or not srcname then
      return
    end

    -- Perform the following ONLY when the player IS in a raid.
    if HTCommon:IsPlayerInRaid() then
      HTBuffMonitor:BuffAnnouncerCombatLogChecker(spellschool, subevent, srcname)
    end

  end

end

-- ============================================================================
-- == BuffAnnouncerCombatLogChecker(spellID, subevent, playerName)           ==
-- ==                                                                        ==
-- == This function determines whether the data supplied by the combat log   ==
-- == event matches a watched-for buff announcement and calls the            ==
-- == appropriate announcement function if it does.                          ==
-- ============================================================================

function HTBuffMonitor:BuffAnnouncerCombatLogChecker(spellID, subevent, playerName)

  -- Return immediately if the Buff Announcer is not enabled.
  if HTConfig:GetIsEnabledBuffAnnouncer(nil) == false then return end

  -- Return immediately if the player doing the action is not in the raid.
  if HTCommon:IsCharacterRaidMember(playerName) == false then return end

  -- Announce Deployment of: Food Buff Dispensers.
  if HTConfig:GetIsEnabledBuffAnnouncerFood(nil) == true then
    if subevent == L["bfm-log-subevent-spell-cast-start"] then
      if HTData:IsInTable(foodBuffDispensers, spellID) then -- tonumber(spellID) == 126504
        HTBuffMonitor:DoBuffAnnouncerFoodAnnouncement(playerName)
        return
      end
    end
  end

  -- Announce Deployment of: Flask Dispensers.
  if HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil) == true then
    if subevent == L["bfm-log-subevent-spell-cast-start"] then
      if HTData:IsInTable(flaskDispensers, spellID) then -- tonumber(spellID) == 126504
        HTBuffMonitor:DoBuffAnnouncerCauldronAnnouncement(playerName)
        return
      end
    end
  end

  -- Announce Deployment of: Health Stone Dispensers.
  if HTConfig:GetIsEnabledBuffAnnouncerHealthStones(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if HTData:IsInTable(healthStoneDispensers, spellID) then -- tonumber(spellID) == 126504
        HTBuffMonitor:DoBuffAnnouncerHealthStonesAnnouncement(playerName)
        return
      end
    end
  end

  -- Announce Deployment of: Mana Food Dispensers.
  if HTConfig:GetIsEnabledBuffAnnouncerFoodTable(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if HTData:IsInTable(manaFoodDispensers, spellID) then -- tonumber(spellID) == 126504
        HTBuffMonitor:DoBuffAnnouncerFoodTableAnnouncement(playerName)
        return
      end
    end
  end

  -- Announce Deployment of: Repair-Bots.
  if HTConfig:GetIsEnabledBuffAnnouncerRepairBot(nil) == true  then
    if subevent == L["bfm-log-subevent-spell-cast-success"]  then
      if HTData:IsInTable(repairBots, spellID) then -- tonumber(spellID) == 126504
        HTBuffMonitor:DoBuffAnnouncerRepairBotAnnouncement(playerName)
        return
      end
    end
  end

end

-- ============================================================================
-- == DoBuffAnnouncerFoodAnnouncement(playerName)                            ==
-- ==                                                                        ==
-- == This function announces the deployment of a food buff.                 ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerFoodAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-food-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-food-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerCauldronAnnouncement(playerName)                        ==
-- ==                                                                        ==
-- == This function announces the deployment of a cauldron.                  ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerCauldronAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-cauldron-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-cauldron-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerHealthStonesAnnouncement(playerName)                    ==
-- ==                                                                        ==
-- == This function announces the deployment of a soulwell.                  ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerHealthStonesAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-healthstones-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-healthstones-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerFoodTableAnnouncement(playerName)                       ==
-- ==                                                                        ==
-- == This function announces the deployment of a food table.                ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerFoodTableAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-foodtable-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-foodtable-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoBuffAnnouncerRepairBotAnnouncement(playerName)                       ==
-- ==                                                                        ==
-- == This function announces the deployment of a repair bot.                ==
-- ============================================================================

function HTBuffMonitor:DoBuffAnnouncerRepairBotAnnouncement(playerName)

  PlaySoundFile("Sound\\interface\\PickUp\\PutDownMeat.wav")
  HTConsole:SendChatMessage(L["bfm-buffannouncer-repairbot-title"], "RAID", nil, nil)
  HTConsole:SendChatMessage(string.gsub(L["bfm-buffannouncer-repairbot-msg-01"], "PLAYER_NAME", playerName), "RAID", nil, nil)

end

-- ============================================================================
-- == DoHeartbeat()                                                          ==
-- ==                                                                        ==
-- == This function is intended to provide a place where non-critical tasks  ==
-- == that need to be performed periodically can be called from. Typically   ==
-- == each HT component may have an instance of this method and each will    ==
-- == be called in turn by the corresponding function in HTConsole which is  ==
-- == considered to be the master-heartbeat component.                       ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeat()

  HTBuffMonitor:DoHeartbeatCheck()

end

-- ============================================================================
-- == DoHeartbeatCheck()                                                     ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame. It is called via the periodic     ==
-- == heartbeat that occurs (normally) every 5-15 minutes depending upon     ==
-- == configuration.                                                         ==
-- ============================================================================

function HTBuffMonitor:DoHeartbeatCheck()

  if HTConfig:GetIsEnabledBuffMonitor(nil) and HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil) then
    HTConsole:SendChatMessage(L["bfm-heartbeatcheck-msg-01"], "CHAT", nil, nil)
    HTBuffMonitor:DoSelfCheck()
    HTConsole:SendChatMessage(L["bfm-heartbeatcheck-msg-02"], "CHAT", nil, nil)
  end

end

-- ============================================================================
-- == DoManualSelfCheck()                                                    ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window and the error-frame.                                   ==
-- ============================================================================

function HTBuffMonitor:DoManualSelfCheck()

  HTConsole:SendChatMessage(L["bfm-manualselfcheck-msg-01"], "CHAT", nil, nil)
  HTBuffMonitor:DoSelfCheck()
  HTConsole:SendChatMessage(L["bfm-manualselfcheck-msg-02"], "CHAT", nil, nil)

end

-- ============================================================================
-- == DoSelfCheck()                                                          ==
-- ==                                                                        ==
-- == This function checks a player's own buffs and reports the findings via ==
-- == the chat window.                                                       ==
-- ============================================================================

function HTBuffMonitor:DoSelfCheck()

  local characterName = UnitName("player")

  HTBuffMonitor:DoMissingBuffCheck(characterName, "CHAT")
  HTBuffMonitor:DoMissingBuffCheck(characterName, "ERRORFRAME")

end

-- ============================================================================
-- == DoRaidCheck(channelName)                                               ==
-- ==                                                                        ==
-- == This function will output the names of raiders that have missing       ==
-- == consumables.                                                           ==
-- ============================================================================

function HTBuffMonitor:DoRaidCheck(channelName)

  -- Return immediately if the player is not in a raid.
  if HTCommon:IsPlayerInRaid() == false then HTConsole:Debug("BuffMonitor - Player must be in a raid to check raid buffs.") return end

  -- Setup the local variables for text strings and data values.
  local raidCheckBeginningText = L["bfm-raidcheck-msg-01"]
  local raidCheckEndingText    = L["bfm-raidcheck-msg-02"]
  local charIsDeadText         = L["bfm-raidcheck-char-is-dead"]
  local charIsOfflineText      = L["bfm-raidcheck-char-is-offline"]

  local numRaidMembers         = HTData:SafeNumber(GetNumGroupMembers())

  -- Process the raid.
  HTConsole:SendChatMessage(raidCheckBeginningText, channelName)

  for index=1, numRaidMembers, 1 do
    local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(index)
    local isCharacterCheckRequired = true

    -- Is the character dead?
    if isDead == 1 then
      if HTConfig:GetIsEnabledBuffMonitorReportDead(nil) then
        HTConsole:SendChatMessage(string.gsub(charIsDeadText, "CHARACTER_NAME", name), channelName, nil, nil)
      end
      isCharacterCheckRequired = false
    end

    -- Is the character offline?
    if online == nil then
      if HTConfig:GetIsEnabledBuffMonitorReportOffline(nil) then
        HTConsole:SendChatMessage(string.gsub(charIsOfflineText, "CHARACTER_NAME", name), channelName, nil, nil)
      end
      isCharacterCheckRequired = false
    end

    -- Check the character, if we still should.
    if isCharacterCheckRequired then
      HTBuffMonitor:DoMissingBuffCheck(name, channelName)
    end

  end

  HTConsole:SendChatMessage(raidCheckEndingText, channelName)

end
